/****************************************************************************
**
** Copyright (C) 2008 Andrey Rijov <ANDron142@yandex.ru>
**
** This file is part of AQEMU.
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor,
** Boston, MA  02110-1301, USA.
**
****************************************************************************/

#include <QProcess>
#include <QRegExp>
#include <QUuid>
#include <QSettings>

#include "VM_Devices.h"
#include "Utils.h"

// VM_Floppy Class ----------------------------------------------------------

VM_Floppy::VM_Floppy()
{
	Enabled = false;
	Host_Device = true;
	Host_File_Name = "";
	Image_File_Name = "";
}

VM_Floppy::VM_Floppy( const VM_Floppy &fd )
{
	Enabled = fd.Get_Enabled();
	Host_File_Name = fd.Get_Host_File_Name();
	Image_File_Name = fd.Get_Image_File_Name();
	Host_Device = fd.Get_Host_Device();
}

VM_Floppy::VM_Floppy( bool enabled, const QString &host_path,
					  const QString &image_path, bool host_device )
{
	Enabled = enabled;
	Host_File_Name = host_path;
	Image_File_Name = image_path;
	Host_Device = host_device;
	
	if( Enabled && Host_Device == false && Image_File_Name.isEmpty() )
	{
		AQWarning( "VM_Floppy::VM_Floppy( bool enabled, const QString &host_path,"
				   "const QString &image_path, bool host_device )",
			       "Image File Name is Empty!" );
	}
}

bool VM_Floppy::operator==( const VM_Floppy &v ) const
{
	if( Enabled == v.Get_Enabled() &&
		   Host_File_Name == v.Get_Host_File_Name() &&
		   Image_File_Name == v.Get_Image_File_Name() &&
		   Host_Device == v.Get_Host_Device() )
	{
		return true;
	}
	else
	{
		return false;
	}
}

bool VM_Floppy::operator!=( const VM_Floppy &v ) const
{
	return ! ( operator==(v) );
}

bool VM_Floppy::Get_Enabled() const
{
	return Enabled;
}

void VM_Floppy::Set_Enabled( bool enabled )
{
	Enabled = enabled;
}

const QString &VM_Floppy::Get_Host_File_Name() const
{
	return Host_File_Name;
}

void VM_Floppy::Set_Host_File_Name( const QString &file_path )
{
	if( file_path.isEmpty() && Enabled )
	{
		AQWarning( "void VM_Floppy::Set_Host_File_Name( const QString &file_path )",
				   "File Name is Empty!" );
		Host_File_Name = "";
	}
	else
	{
		Host_File_Name = file_path;
	}
}

const QString &VM_Floppy::Get_Image_File_Name() const
{
	return Image_File_Name;
}

void VM_Floppy::Set_Image_File_Name( const QString &file_path )
{
	if( file_path.isEmpty() && Enabled )
	{
		AQWarning( "void VM_Floppy::Set_Image_File_Name( const QString &file_path )",
				   "File Name is Empty!" );
		Image_File_Name = "";
	}
	else
	{
		Image_File_Name = file_path;
	}
}

bool VM_Floppy::Get_Host_Device() const
{
	return Host_Device;
}

void VM_Floppy::Set_Host_Device( bool enabled )
{
	Host_Device = enabled;
}

//===========================================================================

// VM_CDROM Class -----------------------------------------------------------

VM_CDROM::VM_CDROM()
{
	Enabled = false;
	Host_File_Name = "";
	Image_File_Name = "";
	Host_Device = true;
}

VM_CDROM::VM_CDROM( const VM_CDROM &cd )
{
	Enabled = cd.Get_Enabled();
	Host_File_Name = cd.Get_Host_File_Name();
	Image_File_Name = cd.Get_Image_File_Name();
	Host_Device = cd.Get_Host_Device();
}

VM_CDROM::VM_CDROM( bool enabled, const QString &host_path,
					const QString &image_path, bool host_device )
{
	Enabled = enabled;
	Host_File_Name = host_path;
	Image_File_Name = image_path;
	Host_Device = host_device;
	
	if( Enabled && Host_Device == false && Image_File_Name.isEmpty() )
	{
		AQWarning( "VM_CDROM::VM_CDROM( bool enabled, const QString &host_path,"
				   "const QString &image_path, bool host_device )",
				   "Image File Name is Empty!" );
	}
}

bool VM_CDROM::operator==( const VM_CDROM &v ) const
{
	if( Enabled == v.Get_Enabled() &&
		Host_File_Name == v.Get_Host_File_Name() &&
		Image_File_Name == v.Get_Image_File_Name() &&
		Host_Device == v.Get_Host_Device() )
	{
		return true;
	}
	else
	{
		return false;
	}
}

bool VM_CDROM::operator!=( const VM_CDROM &v ) const
{
	return ! ( operator==(v) );
}

bool VM_CDROM::Get_Enabled() const
{
	return Enabled;
}

void VM_CDROM::Set_Enabled( bool enabled )
{
	Enabled = enabled;
}

const QString &VM_CDROM::Get_Host_File_Name() const
{
	return Host_File_Name;
}

void VM_CDROM::Set_Host_File_Name( const QString &file_path )
{
	if( file_path.isEmpty() && Enabled )
	{
		AQWarning( "void VM_CDROM::Set_Host_File_Name( const QString &file_path )",
				   "File Name is Empty!" );
		Host_File_Name = "";
	}
	else
	{
		Host_File_Name = file_path;
	}
}

const QString &VM_CDROM::Get_Image_File_Name() const
{
	return Image_File_Name;
}

void VM_CDROM::Set_Image_File_Name( const QString &file_path )
{
	if( file_path.isEmpty() && Enabled )
	{
		AQWarning( "void VM_CDROM::Set_Image_File_Name( const QString &file_path )",
				   "File Name is Empty!" );
		Image_File_Name = "";
	}
	else
	{
		Image_File_Name = file_path;
	}
}

bool VM_CDROM::Get_Host_Device() const
{
	return Host_Device;
}

void VM_CDROM::Set_Host_Device( bool enabled )
{
	Host_Device = enabled;
}

//===========================================================================

// VM_HDD Class -------------------------------------------------------------

VM_HDD::VM_HDD()
{
	Enabled = false;
	Image_File_Name = "";
	
	Disk_Format = "";
	
	VM::Device_Size zero_size;
	zero_size.Size = 0.0;
	zero_size.Suffix = VM::Size_Suf_Gb;
	
	Virtual_Size = zero_size;
	Disk_Size = zero_size;
	
	Cluster_Size = 0;
}

VM_HDD::VM_HDD( const VM_HDD &hd  )
{
	Enabled = hd.Get_Enabled();
	Image_File_Name = hd.Get_Image_File_Name();
	
	if( Image_File_Name.isEmpty() )
	{
		Disk_Format = "";
		
		VM::Device_Size zero_size;
		zero_size.Size = 0.0;
		zero_size.Suffix = VM::Size_Suf_Gb;
		
		Virtual_Size = zero_size;
		Disk_Size = zero_size;
		
		Cluster_Size = 0;
	}
	else
	{
		Update_Image_Info();
	}
}

VM_HDD::VM_HDD( bool enabled, const QString &im_pach )
{
	Enabled = enabled;
	Image_File_Name = im_pach;
	
	if( im_pach.isEmpty() )
	{
		Disk_Format = "";
		
		VM::Device_Size zero_size;
		zero_size.Size = 0.0;
		zero_size.Suffix = VM::Size_Suf_Gb;
		
		Virtual_Size = zero_size;
		Disk_Size = zero_size;
		
		Cluster_Size = 0;
		
		if( Enabled )
		{
			AQWarning( "VM_HDD::VM_HDD( bool enabled, const QString &im_pach )",
					   "File Name is Empty!" );
		}
	}
	else if( Enabled )
	{
		Update_Image_Info();
	}
}

bool VM_HDD::operator==( const VM_HDD &v ) const
{
	if( Enabled == v.Get_Enabled() &&
		Image_File_Name == v.Get_Image_File_Name() )
	{
		return true;
	}
	else
	{
		return false;
	}
}

bool VM_HDD::operator!=( const VM_HDD &v ) const
{
	return ! ( operator==(v) );
}

bool VM_HDD::Get_Enabled() const
{
	return Enabled;
}

void VM_HDD::Set_Enabled( bool enabled )
{
	Enabled = enabled;
}

const QString &VM_HDD::Get_Image_File_Name() const
{
	return Image_File_Name;
}

void VM_HDD::Set_Image_File_Name( const QString &file_name )
{
	if( file_name.isEmpty() && Enabled )
	{
		AQWarning( "void VM_HDD::Set_Image_File_Name( const QString &file_name )",
				   "File Path is Empty!" );
		
		Image_File_Name = "";
		Disk_Format = "";
	
		VM::Device_Size zero_size;
		zero_size.Size = 0.0;
		zero_size.Suffix = VM::Size_Suf_Gb;
	
		Virtual_Size = zero_size;
		Disk_Size = zero_size;
	
		Cluster_Size = 0;
	}
	else
	{
		if( file_name.isEmpty() )
		{
			Image_File_Name = "";
			Disk_Format = "";
	
			VM::Device_Size zero_size;
			zero_size.Size = 0.0;
			zero_size.Suffix = VM::Size_Suf_Gb;
	
			Virtual_Size = zero_size;
			Disk_Size = zero_size;
	
			Cluster_Size = 0;
		}
		else
		{
			Image_File_Name = file_name;
			Update_Image_Info();
		}
	}
}

const QString &VM_HDD::Get_Image_Format() const
{
	return Disk_Format;
}

const VM::Device_Size &VM_HDD::Get_Virtual_Size() const
{
	return Virtual_Size;
}

const VM::Device_Size &VM_HDD::Get_Disk_Size() const
{
	return Disk_Size;
}

int VM_HDD::Get_Cluster_Size() const
{
	return Cluster_Size;
}

double VM_HDD::Get_Virtual_Size_in_GB() const
{
	int div = 0;
	
	switch( Virtual_Size.Suffix )
	{
		case VM::Size_Suf_Kb:
			div = 1048576;
			break;
			
		case VM::Size_Suf_Mb:
			div = 1024;
			break;
			
		case VM::Size_Suf_Gb:
			div = 1;
			break;
			
		default:
			AQError( "double VM_HDD::Get_Virtual_Size_in_GB() const",
					 "Default Section!" );
			return 0.0;
	}
	
	return Virtual_Size.Size / div;
}

QString VM_HDD::Get_Complete_Virtual_Size() const
{
	return QString::number(Virtual_Size.Size) + Size_Suffix_to_String(Virtual_Size.Suffix);
}

QString VM_HDD::Get_Complete_Disk_Size() const
{
	return QString::number(Disk_Size.Size) + Size_Suffix_to_String(Disk_Size.Suffix);
}

void VM_HDD::Set_Virtual_Size( const VM::Device_Size &size )
{
	Virtual_Size = size;
}

void VM_HDD::Set_Image_Format( const QString &format )
{
	Disk_Format = format;
}

QString VM_HDD::Size_Suffix_to_String( VM::Size_Suffix suf ) const
{
	switch( suf )
	{
		case VM::Size_Suf_Kb:
			return "K";
			
		case VM::Size_Suf_Mb:
			return "M";
			
		case VM::Size_Suf_Gb:
			return "G";
			
		default:
			AQError( "QString VM_HDD::Size_Suffix_to_String( VM::Size_Suffix suf ) const",
					 "Size Suffix Incorrect!" );
			return "";
	}
}

VM::Device_Size VM_HDD::String_to_Device_Size( const QString &size ) const
{
	VM::Device_Size zero_size, hd_size;
	zero_size.Size = 0.0;
	zero_size.Suffix = VM::Size_Suf_Gb;
	
	if( size.isEmpty() )
	{
		AQDebug( "VM::Device_Size VM_HDD::String_to_Device_Size( const QString &size ) const",
				 "Size String is Empty..." );
		return zero_size;
	}
	
	QRegExp RegInfo = QRegExp( "([\\d]+|[\\d]+[.]{1}[\\d]+)*([KMG]+)" );
	
	if( ! RegInfo.exactMatch(size) )
	{
		AQError( "VM::Device_Size VM_HDD::String_to_Device_Size( const QString &size ) const",
				 "Cannot Match RegExp!" );
		return zero_size;
	}
	
	QStringList info_lines = RegInfo.capturedTexts();
	
	bool ok = false;
	hd_size.Size = info_lines[1].toDouble( &ok );
	
	if( ! ok )
	{
		AQError( "VM::Device_Size VM_HDD::String_to_Device_Size( const QString &size ) const",
				 "Cannot Convert String to Double!" );
		return zero_size;
	}
	
	if( info_lines[2] == "K" )
	{
		hd_size.Suffix = VM::Size_Suf_Kb;
	}
	else if( info_lines[2] == "M" )
	{
		hd_size.Suffix = VM::Size_Suf_Mb;
	}
	else if( info_lines[2] == "G" )
	{
		hd_size.Suffix = VM::Size_Suf_Gb;
	}
	else
	{
		AQError( "VM::Device_Size VM_HDD::String_to_Device_Size( const QString &size ) const",
				 "HDD Size Suffix Not K,M,G!" );
		return zero_size;
	}
	
	return hd_size;
}

bool VM_HDD::Update_Image_Info()
{
	AQDebug( "bool VM_HDD::Update_Image_Info()", "Begin" );
	
	QProcess *qimg = new QProcess();
	QStringList qimg_args;
	qimg_args << "info" << Image_File_Name;
	
	qimg->start( "qemu-img", qimg_args );
	
	if( ! qimg->waitForFinished(500) )
	{
		AQError( "bool VM_HDD::Update_Image_Info()",
				 "qemu-img Not Finished! Hard Disk Info Cannot Read!" );
		return false;
	}
	
	QByteArray info_str_ba = qimg->readAll();
	
	QString info_str = QString( info_str_ba ); // Create QString
	
	// image:[\s]+(.+).file format:[\s]+([\w\d]+).virtual size:[\s]+([\d]+[.]*[\d]*[KMG]+).*disk size:[\s]+([\d]+[.]*[\d]*[KMG]+).cluster_size:[\s]+([\d]+).*
	QRegExp RegInfo = QRegExp( "image:[\\s]+(.+).file format:[\\s]+([\\w\\d]+).virtual size:[\\s]+([\\d]+[.]*[\\d]*[KMG]+).*disk size:[\\s]+([\\d]+[.]*[\\d]*[KMG]+).cluster_size:[\\s]+([\\d]+).*" );
	
	if( ! RegInfo.exactMatch(info_str) )
	{
		AQError( "bool VM_HDD::Update_Image_Info()",
				 "QRegExp Not Matched!" );
		return false;
	}
	
	QStringList info_lines = RegInfo.capturedTexts();
	
	if( info_lines.count() != 6 )
	{
		AQError( "bool VM_HDD::Update_Image_Info()",
				 "info_str.count() != 6" );
		return false;
	}
	
	if( info_lines[ 1 ] != Image_File_Name )
	{
		AQWarning( "bool VM_HDD::Update_Image_Info()",
				   "info_lines[ 1 ] != Image_File_Name" );
	}
	
	Disk_Format = info_lines[ 2 ];
	Virtual_Size = String_to_Device_Size( info_lines[3] );
	Disk_Size = String_to_Device_Size( info_lines[4] );
	Cluster_Size = info_lines[ 5 ].toInt();
	
	delete qimg;
	
	return true;
}

//===========================================================================

// VM_Net_Card --------------------------------------------------------------

VM_Net_Card::VM_Net_Card()
{
	Card_Model = "";
	Net_Mode = VM::Net_Mode_None;
	IP = "";
	MAC = "";
	Hostname = "";
	Port = 0;
	VLAN = 0;
	Use_TUN_TAP_Script = true;
	TUN_TAP_Script = "";
	Interface_Name = "";
	File_Descriptor = 0;
}

VM_Net_Card::VM_Net_Card( const VM_Net_Card &nc )
{
	Card_Model = nc.Get_Card_Model();
	Net_Mode = nc.Get_Net_Mode();
	Hostname = nc.Get_Hostname();
	IP = nc.Get_IP_Address();
	MAC = nc.Get_MAC_Address();
	Port = nc.Get_Port();
	VLAN = nc.Get_VLAN();
	Use_TUN_TAP_Script = nc.Get_Use_TUN_TAP_Script();
	TUN_TAP_Script = nc.Get_TUN_TAP_Script();
	Interface_Name = nc.Get_Interface_Name();
	File_Descriptor = nc.Get_File_Descriptor();
}

bool VM_Net_Card::operator==( const VM_Net_Card &nc ) const
{
	if( Card_Model == nc.Get_Card_Model() &&
		Net_Mode == nc.Get_Net_Mode() &&
		Hostname == nc.Get_Hostname() &&
		IP == nc.Get_IP_Address() &&
		MAC == nc.Get_MAC_Address() &&
		Port == nc.Get_Port() &&
		VLAN == nc.Get_VLAN() &&
		Use_TUN_TAP_Script == nc.Get_Use_TUN_TAP_Script() &&
		TUN_TAP_Script == nc.Get_TUN_TAP_Script() &&
		Interface_Name == nc.Get_Interface_Name() &&
		File_Descriptor == nc.Get_File_Descriptor() )
	{
		return true;
	}
	else
	{
		return false;
	}
}

bool VM_Net_Card::operator!=( const VM_Net_Card &nc ) const
{
	return ! ( operator==(nc) );
}

QString VM_Net_Card::Generate_MAC() const
{
	QString nmac = QUuid::createUuid();
	nmac = nmac.mid( 25, 12 );
	
	for( int nx = 2; nx < nmac.count(); nx+=3 )
	{
		nmac = nmac.insert( nx, ":" );
	}
	
	return nmac;
}

const QString &VM_Net_Card::Get_Card_Model() const
{
	return Card_Model;
}

void VM_Net_Card::Set_Card_Model( const QString &m )
{
	Card_Model = m;
}

VM::Network_Mode VM_Net_Card::Get_Net_Mode() const
{
	return Net_Mode;
}

void VM_Net_Card::Set_Net_Mode( VM::Network_Mode mode )
{
	Net_Mode = mode;
}

const QString &VM_Net_Card::Get_Hostname() const
{
	return Hostname;
}

void VM_Net_Card::Set_Hostname( const QString &hn )
{
	Hostname = hn;
}

const QString &VM_Net_Card::Get_IP_Address() const
{
	return IP;
}

void VM_Net_Card::Set_IP_Address( const QString &ia )
{
	IP = ia;
}

const QString &VM_Net_Card::Get_MAC_Address() const
{
	return MAC;
}

void VM_Net_Card::Set_MAC_Address( const QString &ma )
{
	MAC = ma;
}

int VM_Net_Card::Get_Port() const
{
	return Port;
}

void VM_Net_Card::Set_Port( int p )
{
	Port = p;
}

int VM_Net_Card::Get_VLAN() const
{
	return VLAN;
}

void VM_Net_Card::Set_VLAN( int vl )
{
	VLAN = vl;
}

bool VM_Net_Card::Get_Use_TUN_TAP_Script() const
{
	return Use_TUN_TAP_Script;
}

void VM_Net_Card::Set_Use_TUN_TAP_Script( bool use )
{
	Use_TUN_TAP_Script = use;
}

const QString &VM_Net_Card::Get_TUN_TAP_Script() const
{
	return TUN_TAP_Script;
}

void VM_Net_Card::Set_TUN_TAP_Script( const QString &s )
{
	TUN_TAP_Script = s;
}

const QString &VM_Net_Card::Get_Interface_Name() const
{
	return Interface_Name;
}

void VM_Net_Card::Set_Interface_Name( const QString &n )
{
	Interface_Name = n;
}

int VM_Net_Card::Get_File_Descriptor() const
{
	return File_Descriptor;
}

void VM_Net_Card::Set_File_Descriptor( int f )
{
	File_Descriptor = f;
}

//===========================================================================

// VM_Redirections Class ----------------------------------------------------

VM_Redirection::VM_Redirection()
{
	Protocol = "NOT";
	Host_Port = 1;
	Guest_IP = "0.0.0.0";
	Guest_Port = 1;
}

VM_Redirection::VM_Redirection( const VM_Redirection &r )
{
	Protocol = r.Get_Protocol();
	Host_Port = r.Get_Host_Port();
	Guest_IP = r.Get_Guest_IP();
	Guest_Port = r.Get_Guest_Port();
}

bool VM_Redirection::operator==( const VM_Redirection &r ) const
{
	if( Protocol == r.Get_Protocol() &&
		Host_Port == r.Get_Host_Port() &&
		Guest_IP == r.Get_Guest_IP() &&
		Guest_Port == r.Get_Guest_Port() )
	{
		return true;
	}
	else
	{
		return false;
	}
}

bool VM_Redirection::operator!=( const VM_Redirection &r ) const
{
	return ! ( operator==(r) );
}

const QString &VM_Redirection::Get_Protocol() const
{
	return Protocol;
}

void VM_Redirection::Set_Protocol( const QString &p )
{
	Protocol = p;
}

int VM_Redirection::Get_Host_Port() const
{
	return Host_Port;
}

void VM_Redirection::Set_Host_Port( int hp )
{
	Host_Port = hp;
}

const QString &VM_Redirection::Get_Guest_IP() const
{
	return Guest_IP;
}

void VM_Redirection::Set_Guest_IP( const QString &ip )
{
	Guest_IP = ip;
}

int VM_Redirection::Get_Guest_Port() const
{
	return Guest_Port;
}

void VM_Redirection::Set_Guest_Port( int p )
{
	Guest_Port = p;
}

//===========================================================================

// VM_Port ------------------------------------------------------------------

VM_Port::VM_Port()
{
	Port_Redir = VM::PR_Default;
	Param_Line = "";
}

VM_Port::VM_Port( const VM_Port &p )
{
	Port_Redir = p.Get_Port_Redirection();
	Param_Line = p.Get_Parametrs_Line();
}

bool VM_Port::operator==( const VM_Port &p ) const
{
	if( Port_Redir == p.Get_Port_Redirection() &&
		Param_Line == p.Get_Parametrs_Line() )
	{
		return true;
	}
	else
	{
		return false;
	}
}

bool VM_Port::operator!=( const VM_Port &p ) const
{
	return ! ( operator==(p) );
}

VM::Port_Redirection VM_Port::Get_Port_Redirection() const
{
	return Port_Redir;
}

void VM_Port::Set_Port_Redirection( VM::Port_Redirection pr )
{
	Port_Redir = pr;
}

const QString &VM_Port::Get_Parametrs_Line() const
{
	return Param_Line;
}

void VM_Port::Set_Parametrs_Line( const QString &pl )
{
	Param_Line = pl;
}

//===========================================================================

// VM_USB Class -------------------------------------------------------------

VM_USB::VM_USB()
{
	Manufacturer_Name = "";
	Product_Name = "";
	Vendor_ID = "";
	Product_ID = "";
}

VM_USB::VM_USB( const VM_USB &u )
{
	Manufacturer_Name = u.Get_Manufacturer_Name();
	Product_Name = u.Get_Product_Name();
	Vendor_ID = u.Get_Vendor_ID();
	Product_ID = u.Get_Product_ID();
}

bool VM_USB::operator==( const VM_USB &u ) const
{
	if( Manufacturer_Name == u.Get_Manufacturer_Name() &&
		Product_Name == u.Get_Product_Name() &&
		Vendor_ID == u.Get_Vendor_ID() &&
		Product_ID == u.Get_Product_ID() )
	{
		return true;
	}
	else
	{
		return false;
	}
}

bool VM_USB::operator!=( const VM_USB &u ) const
{
	return ! ( operator==(u) );
}

const QString &VM_USB::Get_Manufacturer_Name() const
{
	return Manufacturer_Name;
}

void VM_USB::Set_Manufacturer_Name( const QString &name )
{
	Manufacturer_Name = name;
}

const QString &VM_USB::Get_Product_Name() const
{
	return Product_Name;
}

void VM_USB::Set_Product_Name( const QString &name )
{
	Product_Name = name;
}

const QString &VM_USB::Get_Vendor_ID() const
{
	return Vendor_ID;
}

void VM_USB::Set_Vendor_ID( const QString &dev )
{
	Vendor_ID = dev;
}

const QString &VM_USB::Get_Product_ID() const
{
	return Product_ID;
}

void VM_USB::Set_Product_ID( const QString &dev )
{
	Product_ID = dev;
}

QString VM_USB::Get_ID_Line() const
{
	return Vendor_ID + ":" + Product_ID;
}

void VM_USB::Set_ID_Line( const QString &line )
{
	QRegExp id_line = QRegExp( "([\\dabcdef]{4})[:]{1}([\\dabcdef]{4})" );
	
	if( ! id_line.exactMatch(line) )
	{
		AQError( "void VM_USB::Set_ID_Line( const QString &line )",
				 "Cannot Match QRegExp!" );
		return;
	}
	
	QStringList id_str = id_line.capturedTexts();
	
	Vendor_ID = id_str[1];
	Product_ID = id_str[2];
}

//===========================================================================

// VM_USB Class -------------------------------------------------------------

VM_Init_Graphic_Mode::VM_Init_Graphic_Mode()
{
	Enabled = false;
	Width = 800;
	Height = 600;
	Depth = 16;
}

VM_Init_Graphic_Mode::VM_Init_Graphic_Mode( const VM_Init_Graphic_Mode &mode )
{
	Enabled = mode.Get_Enabled();
	Width = mode.Get_Width();
	Height = mode.Get_Height();
	Depth = mode.Get_Depth();
}

bool VM_Init_Graphic_Mode::operator==( const VM_Init_Graphic_Mode &mode ) const
{
	if( Enabled == mode.Get_Enabled() &&
		Width == mode.Get_Width() &&
		Height == mode.Get_Height() &&
		Depth == mode.Get_Depth() )
	{
		return true;
	}
	else
	{
		return false;
	}
}

bool VM_Init_Graphic_Mode::operator!=( const VM_Init_Graphic_Mode &mode ) const
{
	return ! ( operator==(mode) );
}

bool VM_Init_Graphic_Mode::Get_Enabled() const
{
	return Enabled;
}

void VM_Init_Graphic_Mode::Set_Enabled( bool use )
{
	Enabled = use;
}

int VM_Init_Graphic_Mode::Get_Width() const
{
	return Width;
}

void VM_Init_Graphic_Mode::Set_Width( int w )
{
	Width = w;
}

int VM_Init_Graphic_Mode::Get_Height() const
{
	return Height;
}

void VM_Init_Graphic_Mode::Set_Height( int h )
{
	Height = h;
}

int VM_Init_Graphic_Mode::Get_Depth() const
{
	return Depth;
}

void VM_Init_Graphic_Mode::Set_Depth( int d )
{
	Depth = d;
}

//===========================================================================
